package big.epfl.ch.doppler.gui;

import static big.epfl.ch.doppler.objects.Color.WHITE;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;

import processing.core.PApplet;
import processing.core.PFont;
import processing.core.PGraphics;
import processing.core.PImage;
import processing.video.Movie;
import big.epfl.ch.doppler.imageprocessing.BloodFlow;
import big.epfl.ch.doppler.imageprocessing.Deinterlacing;
import big.epfl.ch.doppler.objects.Color;
import big.epfl.ch.doppler.objects.Image;
import big.epfl.ch.doppler.objects.Point;
import big.epfl.ch.doppler.utilities.Smoother;

public class Test extends PApplet {

	//added vars and const
	private PFont font;
	private static String filepath;
	private List<Float> angles;
	public static float MAX_FLOW;
	private static int PX_PER_CM;
	private static int SCALE_FACTOR;
	private FileWriter writer;
	private static float threshold;
	
	private Image arg;
	private Image[] deinterlacedImgs;
	private PImage img;
	private Movie movie;

	private Deinterlacing deinterlacing;
	private List<Point> corners;
	private boolean start, pause;

	private PGraphics graph;
	private Map<Integer, Float> colorMap;
	private List<Float> bloodFlow;
	private BloodFlow[] bloods;
	private Thread[] threads;
	
	private boolean usingSnake = false;
	
	
	public static void main(String[] args){
		PApplet.main(Test.class.getName());
		
		try{
			filepath = args[0];
			MAX_FLOW = Integer.parseInt(args[1]);
			PX_PER_CM = Integer.parseInt(args[2]);
			threshold = Float.parseFloat(args[3]);
			SCALE_FACTOR = PX_PER_CM*PX_PER_CM/4;
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("Usage: $java -jar <jar filepath> <video filepath> <max speed> <scale px/cm>");
			System.exit(1);
		}
	}
	
	public void settings(){
		
		size(720, 576, FX2D);
	}

	public void setup(){
		//added config
		surface.setResizable(true);
		font = createFont("Arial Bold", 20, false);
		textFont(font);
		fill(WHITE);

		movie = new Movie(this, filepath);
		
		movie.loop();
		movie.read();
		
		while(!(movie.get().width > 0 && movie.get().height > 0)){
			movie.read();
		}
		img = movie.get();
		
		arg = new Image(img.width, img.height, img.pixels);

		surface.setSize(img.width, img.height);
		
		image(img, 0, 0);

		
		try {
			Path p = Paths.get(filepath);
			String filename = p.getFileName().toString();
			filename = filename.substring(0, filename.length() - 4);
			File f = new File("./results/"+filename+".csv");
			f.getParentFile().mkdirs();
			writer = new FileWriter(f);
			writer.append("Angle");
			writer.append(",");
			writer.append("Flow");
			writer.append("\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		colorMap = new HashMap<Integer,Float>();
		corners = new ArrayList<Point>();
		graph = createGraphics(3*img.width/2, img.height/4);
		bloodFlow = new ArrayList<Float>();
		angles = new ArrayList<Float>();
	}

	public void draw(){
		
		if(pause) {
			movie.pause();
		}
		else {
			movie.play();
			movie.read();
			img = movie.get();
			
			image(img, 0, 0);
			
			if(start){	

				arg.setPixels(img.pixels);
				deinterlacedImgs = deinterlacing.deinterlace(arg);
				
				// compute blood flow 				
				bloods = new BloodFlow[deinterlacedImgs.length];
				threads = new Thread[bloods.length];

				for(int i=0; i<deinterlacedImgs.length; i++){
					bloods[i] = new BloodFlow(deinterlacedImgs[i], colorMap, usingSnake);
					threads[i] = new Thread(bloods[i]);
				}
				for(int i=0; i<deinterlacedImgs.length; i++){
					threads[i].start();;
				}
				for(int i=0; i<deinterlacedImgs.length; i++){
					try {
						threads[i].join();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				for(int i=0; i<deinterlacedImgs.length; i++){
					bloodFlow.add(bloods[i].getFlow());
					angles.add(bloods[i].getAngle());
				}
				drawInterface();
				try {
					writer.append(String.valueOf(Average(angles)));
					writer.append(",");
					writer.append(String.valueOf(4*60*(100*Average(bloodFlow)/SCALE_FACTOR)/100f));
					writer.append("\n");
					
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				if(usingSnake){
					drawSnakes();
				}
			}
			else {
				for(Point c : corners){
					fill(255, 128, 0);
					ellipse(c.getX(), c.getY(), 10, 10);
				}
			}
		}
		
	}

	public void drawInterface(){

		BloodFlow blood = new BloodFlow();
		// display the small images
		for(int i = 0; i<deinterlacedImgs.length; i++){
			// deinterlaced images
			display(deinterlacedImgs[i], img.width, i*img.height/4);
			// biggest bloodstain
			display(blood.getLCC(blood.getBlood(deinterlacedImgs[i]), false), 5*img.width/4, i*img.height/4);
		}
		// separating all the displayed images
		stroke(255);
		strokeWeight(1.5f);
		line(img.width, 0, img.width, img.height);
		line(5*img.width/4, 0, 5*img.width/4, img.height);
		line(img.width, img.height/4, 3*img.width/2, img.height/4);
		line(img.width, img.height/2, 3*img.width/2, img.height/2);
		line(img.width, 3*img.height/4, 3*img.width/2, 3*img.height/4);
		
		drawGraph();
		drawText();
	}
	
	/**@Raoul
	 * Draws the measurements on the frame.
	 * For each data type, smooths by averaging on the last values 
	 */
	public void drawText(){
		int lineheight = 20;
		int lmargin = 5;
		textFont(font, 15);
		text("Angle", lmargin, img.height-3*lineheight);
		text("Flow [ml/min]", lmargin, img.height-6*lineheight);
		
		textFont(font, 45);
		Smoother smangle = new Smoother(angles);
		Smoother smflow = new Smoother(bloodFlow);
		text(String.format("%02.0f", smangle.smooth())+"\u00b0", lmargin, img.height-lineheight);
		text(String.format("%3.0f", 4*60*(Math.round(100*smflow.smooth()/SCALE_FACTOR))/100f), lmargin, img.height-4*lineheight);
	}

	public void drawGraph() {

		graph.beginDraw();
		graph.background(0);
		graph.stroke(255);
		graph.strokeWeight(1.5f);
		graph.line(0, 0, 3*img.width/2, 0);

		int ecart = 100;
		graph.line(graph.width-ecart, 0, graph.width-ecart, graph.height);

		int beginX = max(0, bloodFlow.size()-graph.width)+ecart;

		ecart -= 10;

		float maxFlow = 0, minFlow = 0;
		for (int i = beginX; i < bloodFlow.size(); i++) {
			float flow = bloodFlow.get(i);
			maxFlow = max(flow, maxFlow);
			minFlow = min(flow, minFlow);
		}
		float scale = 2*max(maxFlow, -minFlow)/graph.height;
		for(int i = beginX; i < bloodFlow.size()-1; i++) {
			float flow = bloodFlow.get(i);
			float nextFlow = bloodFlow.get(i+1);
			graph.stroke(255);
			graph.line(i-beginX, graph.height/2-flow/scale, i+1-beginX, graph.height/2-nextFlow/scale);
		}
		graph.text((Math.round(4*100*max(maxFlow, -minFlow)/SCALE_FACTOR))/100f + " mL/s", graph.width-ecart, 15);
		graph.text(0 + " mL/s", graph.width-ecart, graph.height/2);
		graph.text((Math.round(4*100*min(-maxFlow, minFlow)/SCALE_FACTOR))/100f + " mL/s", graph.width-ecart, graph.height-5);

		graph.endDraw();
		image(graph, 0, img.height);
	}
	
	public void drawSnakes(){
		for(int i=0; i<deinterlacedImgs.length; i++){
			Point[] snakePoints = bloods[i].getSnakePoints();
			for(int j = 0; j<snakePoints.length-1; j++){
				stroke(255);
				line(snakePoints[j].getX()+5*img.width/4, snakePoints[j].getY()+i*deinterlacedImgs[i].height, 
						snakePoints[j+1].getX()+5*img.width/4, snakePoints[j+1].getY()+i*deinterlacedImgs[i].height);
			}
		}
	}

	/**
	 * display an image of type Image at location x and y
	 */
	public void display(Image img, int x, int y){
		PImage toDisplay = createImage(img.width, img.height, RGB);
		for(int i = 0; i<img.size(); i++){
			toDisplay.pixels[i] = img.pixels[i];
		}
		image(toDisplay, x, y);
	}

	@Override
	public void keyPressed(){
		if(key == ENTER){
			start = true;
			arg.setPixels(img.pixels);
			if(corners.isEmpty()){
				deinterlacing = new Deinterlacing(arg);
			}
			else deinterlacing = new Deinterlacing(corners, new Point(img.width/2, img.height/2));
			corners = deinterlacing.getCorners();

			Color.fillColorMap(threshold, colorMap, arg, corners);
			
			surface.setSize(3*img.width/2, 5*img.height/4);
		}
		if(key == TAB){
			pause = !pause;
		}
		if(key == ESC){
			key=0;
		    try {
				writer.flush();
				writer.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		    exit();
		}
	}

	@Override
	public void mousePressed(){
		if(!start){
			corners.add(new Point(pmouseX, pmouseY));
		}
	}
	
	private float Average(List<Float> l) {
		OptionalDouble average = l
	            .stream()
	            .mapToDouble(a -> a)
	            .average();
		
		return (float) average.orElse( 0 );
	}
}