package big.epfl.ch.doppler.objects;

import java.util.List;

/**
 * @author Axel Vandebrouck & Brune Bastide
 * 
 */
public class Point {

	private int x, y;
	
	/**
	 * @param x : the x coordinate of the point
	 * @param y : the y coordinate of the point
	 */
	public Point(int x, int y){
		this.x = x;
		this.y = y;
	}
	
	/**
	 * @return the x coordinate of the point
	 */
	public int getX(){
		return x;
	}
	
	/**
	 * @return the y coordinate of the point
	 */
	public int getY(){
		return y;
	}
	
	/**
	 * @param newX : the new x coordinate of the point
	 * @param newY : the new y coordinate of the point
	 */
	public void setLocation(int newX, int newY) {
		this.x = newX;
		this.y = newY;
	}

	/**
	 * @param that : a point which we want to compute the distance with the currant point
	 * @return the euclidian distance between the currant point and that
	 */
	public float dist(Point that) {
		float deltaX = that.getX()-this.x;
		float deltaY = that.getY()-this.y; 
		return (float)(Math.sqrt(deltaX*deltaX + deltaY*deltaY));
	}
	
	@Override
	public String toString(){
		return "("+x+", "+y+")";
	}
	
	/**
	 * @param corners : a list of points forming a polygon
	 * @return true if the currant point is inside the polygon 
	 */
	public boolean isInside(List<Point> corners){
		int idx = 0;
		for(int i = 0; i < corners.size(); i++){
			Point p1 = corners.get(i);
			Point p2 = corners.get((i+1)%corners.size());
			if(p1.getY() <= this.y){
				if(p2.getY() > this.y && isLeft(p1, p2)){
					idx++;
				}
			}
			else {
				if(p2.getY() <= this.y && isLeft(p2, p1)){
					idx--;
				}
			}
		}
		return idx != 0;
	}
	
	private boolean isLeft(Point p1, Point p2){
		return (p1.getX() - this.x)*(p2.getY() - this.y) > (p2.getX() - this.x)*(p1.getY() - this.y);
	}
}