package big.epfl.ch.doppler.preprocessing;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import big.epfl.ch.doppler.objects.Point;

/**
 * 
 * @author Axel Vandebrouck & Brune Bastide
 */
public class Window {
	
	private List<Point> corners;
	
	/**
	 * @param the corners of the green parallelogram 
	 * @throws IllegalArgumentException if the list of corners is empty
	 */
	public Window(List<Point> corners) throws IllegalArgumentException {
		if(corners.isEmpty()){
			throw new IllegalArgumentException("The list of corners is empty");
		}
		this.corners = corners;
	}
	
	/**
	 * @param the center of the parallelogram 
	 * @return the corners sorted in clockwise order with the upper left corner in first position
	 */
	public List<Point> sortCorners(Point center){
		// Sort corners so that they are ordered clockwise
		if(corners.size() > 2){
			Collections.sort(corners, new CWComparator(center));
			// Re-order the corners so that the first one is the closest to the origin (0,0) of the image.
			int idx = 0;
			Point origin = new Point(0, 0);
			float smallestDistance = corners.get(0).dist(origin);
			for(int i=1; i<corners.size(); i++){
				Point c = corners.get(i);	
				float dist = c.dist(origin);
				if(dist < smallestDistance){
					smallestDistance = dist;
					idx = i;
				}
			}
			Collections.rotate(corners, idx);
		}
		// delete the green frame
		return corners;
	}
	
	class CWComparator implements Comparator<Point> { 

		Point center;
		public CWComparator(Point center) {
			this.center = center; 
		}
		@Override
		public int compare(Point b, Point d) { 
			if(Math.atan2(b.getY()-center.getY(),b.getX()-center.getX())<Math.atan2(d.getY()-center.getY(),d.getX()-center.getX()))
				return -1; 
			else return 1;
		}
	}
}