package big.epfl.ch.doppler.imageprocessing;

import static big.epfl.ch.doppler.objects.Color.WHITE;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;

import big.epfl.ch.doppler.objects.Color;
import big.epfl.ch.doppler.objects.Image;
import big.epfl.ch.doppler.objects.Point;
import big.epfl.ch.doppler.utilities.Smoother;
import processing.core.PApplet;
import processing.core.PFont;
import processing.core.PGraphics;
import processing.core.PImage;
import processing.video.Movie;

public class TestImage extends PApplet{
	
	private String path;
	private final int WIDTH = 720, HEIGHT = 576;
	private PImage img;
	private PFont font;
	private Image arg;
	public static float MAX_SPEED = 10;
	private static int SCALE_FACTOR = 625;
	
	private List<Point> corners;
	private PGraphics graph;
	private Map<Integer, Float> colorMap;
	private List<Float> bloodFlow;
	private List<Float> angles;
	private List<Integer> areas;
	private List<Float> speeds;
	
	private Image[] deinterlacedImgs;
	private Deinterlacing deinterlacing;
	private BloodFlow[] bloods;
	private Thread[] threads;

	public TestImage(String path){
		this.path = path;
		this.runSketch();
	}
	
	public void settings(){
		
		//size(720, 576);
	}
	
	public void setup(){
		
		surface.setResizable(true);
		font = createFont("Arial Bold", 20, false);
		textFont(font);
		fill(WHITE);
		
		img = loadImage(path);
		arg = new Image(img.width, img.height, img.pixels);
		graph = createGraphics(3*img.width/2, img.height/4);
		bloodFlow = new ArrayList<Float>();
		angles = new ArrayList<Float>();
		speeds = new ArrayList<Float>();
		areas = new ArrayList<Integer>();
		colorMap = new HashMap<Integer,Float>();
		corners = new ArrayList<Point>();
		//set corners corresponding to the green rectangle of test images
		corners.add(new Point(160, 10));
		corners.add(new Point(160, 566));
		corners.add(new Point(710, 566));
		corners.add(new Point(710, 10));
	}
	
	public void draw(){
		
		//image(img, 0, 0);
		
		arg.setPixels(img.pixels);
		
		deinterlacing = new Deinterlacing(corners, new Point(img.width/2, img.height/2));
		corners = deinterlacing.getCorners();

		Color.fillColorMap(colorMap, arg, corners, MAX_SPEED);

		surface.setSize(3*img.width/2, 5*img.height/4);
		
		/*for(Point c : corners){
			fill(255, 128, 0);
			ellipse(c.getX(), c.getY(), 10, 10);
		}*/
		
		arg.setPixels(img.pixels);
		deinterlacedImgs = deinterlacing.deinterlace(arg);
				
		// compute blood flow 				
		bloods = new BloodFlow[deinterlacedImgs.length];
		threads = new Thread[bloods.length];

		for(int i=0; i<deinterlacedImgs.length; i++){
			bloods[i] = new BloodFlow(deinterlacedImgs[i], colorMap, false);
			threads[i] = new Thread(bloods[i]);
		}
		for(int i=0; i<deinterlacedImgs.length; i++){
			threads[i].start();;
		}
		for(int i=0; i<deinterlacedImgs.length; i++){
			try {
				threads[i].join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
		for(int i=0; i<deinterlacedImgs.length; i++){
			if(angles.size()<4){
			//bloodFlow.add(bloods[i].getFlow());
				angles.add(bloods[i].getAngle());
				areas.add(bloods[i].getVesselarea());
				speeds.add(bloods[i].getSpeed());
			}
		}
		//System.out.println(angles.toString());
		//drawInterface();
	}
	
	public void drawInterface(){

		BloodFlow blood = new BloodFlow();
		// display the small images
		for(int i = 0; i<deinterlacedImgs.length; i++){
			// deinterlaced images
			display(deinterlacedImgs[i], img.width, i*img.height/4);
			// biggest bloodstain
			display(blood.getLCC(blood.getBlood(deinterlacedImgs[i]), false), 5*img.width/4, i*img.height/4);
		}
		// separating all the displayed images
		stroke(255);
		strokeWeight(1.5f);
		line(img.width, 0, img.width, img.height);
		line(5*img.width/4, 0, 5*img.width/4, img.height);
		line(img.width, img.height/4, 3*img.width/2, img.height/4);
		line(img.width, img.height/2, 3*img.width/2, img.height/2);
		line(img.width, 3*img.height/4, 3*img.width/2, 3*img.height/4);
	}
	
	/**
	 * display an image of type Image at location x and y
	 */
	public void display(Image img, int x, int y){
		PImage toDisplay = createImage(img.width, img.height, RGB);
		for(int i = 0; i<img.size(); i++){
			toDisplay.pixels[i] = img.pixels[i];
		}
		image(toDisplay, x, y);
	}

	public float getBloodFlowAverage() {
		OptionalDouble average = bloodFlow
	            .stream()
	            .mapToDouble(a -> a)
	            .average();
		
		return (float)average.orElse( 0 );
	}
	
	public float getAngleAverage() {
		OptionalDouble average = angles
	            .stream()
	            .mapToDouble(a -> a)
	            .average();
		
		return (float)average.orElse( 0 );
	}
	
	public float getAreasAverage() {
		OptionalDouble average = areas
	            .stream()
	            .mapToDouble(a -> a)
	            .average();
		
		return (float)average.orElse( 0 );
	}
	
	public float getSpeedsAverage() {
		OptionalDouble average = speeds
	            .stream()
	            .mapToDouble(a -> a)
	            .average();
		
		return (float) average.orElse( 0 );
	}
	
	public List<Float> getAngles() {
		return angles;
	}
	
	public List<Integer> getAreas() {
		return areas;
	}
	
	public List<Float> getSpeeds() {
		return speeds;
	}
}
